RGO = LibStub("AceAddon-3.0"):NewAddon("RGO", "AceEvent-3.0")
RGO_Console = LibStub("AceConsole-3.0")

function RGO:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RGO_DB")
	
	if(self.db.realm.presetIndexToGroupStructure == nil) then
		self.db.realm.presetIndexToGroupStructure = {}
	end
	
	if(self.db.realm.presetIndexToPresetName == nil) then 
		self.db.realm.presetIndexToPresetName = {}
	end
	
	RGO:RegisterEvent("GROUP_ROSTER_UPDATE", "UpdateDropdownVisibility")
	RGO:RegisterEvent("PLAYER_ROLES_ASSIGNED", "UpdateDropdownVisibility")
	RGO:RegisterEvent("PARTY_LEADER_CHANGED", "UpdateDropdownVisibility")
	
	RGO:CreatePresetDropdownInRaidFrame() 
	
	RGO:UpdateDropdownVisibility() 
end

function RGO:OnEnable()
end

function RGO:OnDisable()
end

function RGO:CreatePresetDropdownInRaidFrame() 
	self.presetDropDown = CreateFrame("FRAME", "RGOChoosePreset", RaidFrame, "UIDropDownMenuTemplate")
	self.presetDropDown:SetPoint("TOPLEFT", RaidFrame, "TOPLEFT", 102, -22)
	RGOChoosePresetRight:SetAlpha(0)
	RGOChoosePresetLeft:SetAlpha(0)
	RGOChoosePresetMiddle:SetAlpha(0)
	UIDropDownMenu_SetWidth(self.presetDropDown, 7)

	local function presetDropDown_OnClick(frame, arg1)
		RGO:sortGroup(self.db.realm.presetIndexToGroupStructure[arg1])
	end
	
	UIDropDownMenu_Initialize(self.presetDropDown, function(frame, level, menuList)
		local info = UIDropDownMenu_CreateInfo()
		
		for i = 1, #self.db.realm.presetIndexToPresetName do
			info.func = presetDropDown_OnClick
			info.text = self.db.realm.presetIndexToPresetName[i]
			info.arg1 = i
			UIDropDownMenu_AddButton(info)
		end
	end)
end

function RGO:UpdateDropdownVisibility() 
	if UnitInRaid("player")and not InCombatLockdown() and ((UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")) and RGO:getPresetCount() > 0) then
		self.presetDropDown:Show()
	else
		self.presetDropDown:Hide()
	end
end

function RGO:deletePreset(index) 
	if(index ~= nil) then 
		table.remove(self.db.realm.presetIndexToPresetName, index)
		table.remove(self.db.realm.presetIndexToGroupStructure, index)
	end
	
	if(RGO:getPresetCount() == 0) then
		RGO:UpdateDropdownVisibility() 
	end
end

function RGO:addNewPreset(name, group) 
	table.insert(self.db.realm.presetIndexToPresetName, name)
	table.insert(self.db.realm.presetIndexToGroupStructure, group)
	
	if(RGO:getPresetCount() == 1) then
		RGO:UpdateDropdownVisibility()
	end
end

function RGO:updatePreset(index, name, group) 
	self.db.realm.presetIndexToPresetName[index] = name
	self.db.realm.presetIndexToGroupStructure[index] = group
end

function RGO:getPresetCount()
	return #self.db.realm.presetIndexToPresetName
end
function RGO:getPresetName(index)
	return self.db.realm.presetIndexToPresetName[index]
end

function RGO:getPresetGroup(index)
	return self.db.realm.presetIndexToGroupStructure[index]
end

function RGO:sortGroup(sortedGRP)
	
	local indexToName = {}
	local indexToSubGrp = {}
	local nameToIndex = {}
	
	for i = 1, 40 do
		indexToName[i], a, indexToSubGrp[i] = GetRaidRosterInfo(i)
		if(indexToName[i] ~= nil) then
			nameToIndex[indexToName[i]] = i
		else 
			indexToSubGrp[i] = nil
		end
	end  
	
	local subGrpSize = {0,0,0,0,0,0,0,0}
	for i = 1, 40 do
		if indexToSubGrp[i] ~= nil then
			subGrpSize[indexToSubGrp[i]] = subGrpSize[indexToSubGrp[i]] + 1
		end
	end  

	for i = 1, 8 do	
		for j = 1, 5 do
			local currentIndex = (i - 1) * 5 + j 
			if(nameToIndex[sortedGRP[currentIndex]] ~= nil) then
				local actuallIndex = nameToIndex[sortedGRP[currentIndex]]
				
				if(indexToSubGrp[actuallIndex] == i) then
					indexToSubGrp[actuallIndex] = 0
				elseif(subGrpSize[i] < 5) then
					subGrpSize[indexToSubGrp[actuallIndex]] = subGrpSize[indexToSubGrp[actuallIndex]] -1
					
					SetRaidSubgroup(actuallIndex, i)
					
					indexToSubGrp[actuallIndex] = 0
					subGrpSize[i] = subGrpSize[i] + 1
				else 
					for i2 = 1, 40 do
						if(indexToSubGrp[i2] == i) then
							SwapRaidSubgroup(i2, actuallIndex)
							indexToSubGrp[i2] = indexToSubGrp[actuallIndex]
							break
						end
					end
					indexToSubGrp[actuallIndex] = 0
				end
			end
		end
	end
end


RGO_Console:RegisterChatCommand("rgo", "CreateRGOWindow")

function RGO_Console:CreateRGOWindow(input)
    if not RgoFrame:IsVisible() then
        RgoFrame:Show()
    end
end

