RGO = LibStub("AceAddon-3.0"):NewAddon("RGO")
RGO_Console = LibStub("AceConsole-3.0")
RGO_GUI = LibStub("AceGUI-3.0")

function RGO:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RGO_DB")
	
	self.mainFrame = nil
	self.selected = nil
	
	if(self.db.realm.presetIndexToGroupStructure == nil) then
		self.db.realm.presetIndexToGroupStructure = {}
	end
	
	if(self.db.realm.presetIndexToPresetName == nil) then 
		self.db.realm.presetIndexToPresetName = {}
	end
	
	local presetDropDown = CreateFrame("FRAME", "RGOChoosePreset", RaidFrame, "UIDropDownMenuTemplate")
	presetDropDown:SetPoint("TOPLEFT", RaidFrame, "TOPRIGHT", -13, 3)
	UIDropDownMenu_SetWidth(presetDropDown, 50)
	UIDropDownMenu_SetText(presetDropDown, "Sort")
	
	local function presetDropDown_OnClick(frame, arg1)
		RGO:sortGroup(self.db.realm.presetIndexToGroupStructure[arg1])
	end
	
	UIDropDownMenu_Initialize(presetDropDown, function(frame, level, menuList)
		local info = UIDropDownMenu_CreateInfo()
		for i = 1, #self.db.realm.presetIndexToPresetName do
			info.func = presetDropDown_OnClick
			info.text = self.db.realm.presetIndexToPresetName[i]
			info.arg1 = i
			UIDropDownMenu_AddButton(info)
		end
	end)
	
	local eventManager = CreateFrame("FRAME", "EventManager")
	presetDropDown:RegisterEvent("GROUP_ROSTER_UPDATE")
	presetDropDown:RegisterEvent("PLAYER_ROLES_ASSIGNED")
	presetDropDown:RegisterEvent("PARTY_LEADER_CHANGED")
	presetDropDown:SetScript("OnEvent",
	function(self, event, ...)
		if UnitInRaid("player")and not InCombatLockdown() and (UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")) then
			presetDropDown:Show()
		else
			presetDropDown:Hide()
		end
	end)
end

function RGO:OnEnable()
	
end

function RGO:OnDisable()
    -- Called when the addon is disabled
end

function RGO:IsOpen()
    return self.mainFrame and self.mainFrame:IsVisible()
end

function RGO:closeWindow()
    RGO_GUI:Release(self.mainFrame)
end

function RGO:CreateWindow()

	self.mainFrame = RGO_GUI:Create("Frame")
	self.mainFrame:SetTitle("Raid Group Organizer")
	self.mainFrame:SetStatusText("Raid Group Organizer")
	self.mainFrame:SetCallback("OnClose", function(widget) RGO_GUI:Release(widget) end)
	self.mainFrame:SetLayout("Flow")
	self.mainFrame:SetWidth(600)
	self.mainFrame:SetHeight(600)
	self.mainFrame:EnableResize(false)
	
	local sideBar = RGO_GUI:Create("SimpleGroup")
	sideBar:SetRelativeWidth(0.3)
	sideBar:SetFullHeight(true)
	sideBar:SetLayout("Flow")
	
	local newButton = RGO_GUI:Create("Button")
	newButton:SetText("New")
	newButton:SetWidth(80)
	
	local deleteButton = RGO_GUI:Create("Button")
	deleteButton:SetText("Delete")
	deleteButton:SetWidth(80)
	
	local scrollcontainer = RGO_GUI:Create("InlineGroup") 
	scrollcontainer:SetFullWidth(true)
	scrollcontainer:SetFullHeight(true) 
	scrollcontainer:SetLayout("Fill") 
	
	local scroll = RGO_GUI:Create("ScrollFrame")
	scroll:SetLayout("List") 
	
	local groupListGroup = RGO_GUI:Create("SimpleGroup")
	groupListGroup:SetRelativeWidth(0.7)
	groupListGroup:SetHeight(120) --?????????
	groupListGroup:SetLayout("Flow")
	
	
	newButton:SetCallback("OnClick",function() RGO:addNewPreset(scroll, groupListGroup) end)
	deleteButton:SetCallback("OnClick",function()  RGO:deletePreset(self.selected, scroll, groupListGroup) end)
	
	self.mainFrame:AddChild(sideBar)
	sideBar:AddChild(newButton)	
	sideBar:AddChild(deleteButton)
	sideBar:AddChild(scrollcontainer)
	scrollcontainer:AddChild(scroll)
	
	for i = 1, #self.db.realm.presetIndexToPresetName do
		scroll:AddChild(RGO:CreatePresetLable(i, self.db.realm.presetIndexToPresetName[i], groupListGroup))
	end 
	
	self.mainFrame:AddChild(groupListGroup)
end

function RGO:deletePreset(index, scroll, groupListGroup) 
	if(index ~= nil) then 
		groupListGroup:ReleaseChildren()
		table.remove(self.db.realm.presetIndexToPresetName, index)
		table.remove(self.db.realm.presetIndexToGroupStructure, index)
		scroll:ReleaseChildren()
		for i = 1, #self.db.realm.presetIndexToPresetName do
			scroll:AddChild(RGO:CreatePresetLable(i, self.db.realm.presetIndexToPresetName[i], groupListGroup))
		end 
	end
end

function RGO:addNewPreset(scroll, groupListGroup) 
	table.insert(self.db.realm.presetIndexToPresetName, "Preset " .. (#self.db.realm.presetIndexToPresetName + 1))
	table.insert(self.db.realm.presetIndexToGroupStructure, {})
	scroll:AddChild(RGO:CreatePresetLable(#self.db.realm.presetIndexToPresetName ,self.db.realm.presetIndexToPresetName[#self.db.realm.presetIndexToPresetName], groupListGroup))
end

function RGO:CreatePresetLable(index, presetName, groupListGroup) 
	local presetLableBox = RGO_GUI:Create("SimpleGroup")
	presetLableBox:SetFullWidth(true)

	local text = RGO_GUI:Create("InteractiveLabel")
	text:SetText(presetName)
	text:SetHighlight("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	text:SetFont(GameFontHighlightSmall:GetFont(), 15)
	text:SetCallback("OnClick",
		function() 
			self.selected = index
			groupListGroup:ReleaseChildren()
			RGO:AddGroupLayout(groupListGroup, index) 
		end
	)
	presetLableBox:AddChild(text)
	
	return presetLableBox
end

function RGO:AddGroupLayout(groupListGroup, index)
	for i = 1, 8 do 
		groupListGroup:AddChild(RGO:CreateGroup(i, index))
	end
end

function RGO:CreateGroup(grpNum, index) 
	local group = RGO_GUI:Create("InlineGroup")
	group:SetTitle("Group" .. grpNum)
	group:SetWidth(175)
	
	local player1 = RGO_GUI:Create("EditBox")
	if(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 1] ~= nil) then
		player1:SetText(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 1]);
	end
	player1:SetWidth(150)
	player1:SetHeight(11)
	player1:SetCallback("OnEnterPressed", function(widget, event, text) self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 1] = text end)
	group:AddChild(player1)
	
	local player2 = RGO_GUI:Create("EditBox")
	if(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 2] ~= nil) then
		player2:SetText(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 2]);
	end
	player2:SetWidth(150)
	player2:SetCallback("OnEnterPressed", function(widget, event, text) self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 2] = text end)
	group:AddChild(player2)
	
	local player3 = RGO_GUI:Create("EditBox")
	if(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 3] ~= nil) then
		player3:SetText(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 3]);
	end
	player3:SetWidth(150)
	player3:SetHeight(11)
	player3:SetCallback("OnEnterPressed", function(widget, event, text) self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 3] = text end)
	group:AddChild(player3)
	
	local player4 = RGO_GUI:Create("EditBox")
	if(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 4] ~= nil) then
		player4:SetText(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 4]);
	end
	player4:SetWidth(150)
	player4:SetCallback("OnEnterPressed", function(widget, event, text) self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 4] = text end)
	group:AddChild(player4)
	
	local player5 = RGO_GUI:Create("EditBox")
	if(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 5] ~= nil) then
		player5:SetText(self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 5]);
	end
	player5:SetWidth(150)
	player5:SetHeight(11)
	player5:SetCallback("OnEnterPressed", function(widget, event, text) self.db.realm.presetIndexToGroupStructure[index][(grpNum - 1) * 5 + 5] = text end)
	group:AddChild(player5)
	
	return group
end

function RGO:sortGroup(sortedGRP)
	
	local indexToName = {}
	local indexToSubGrp = {}
	local nameToIndex = {}
	
	for i = 1, 40 do
		indexToName[i], a, indexToSubGrp[i] = GetRaidRosterInfo(i)
		if(indexToName[i] ~= nil) then
			nameToIndex[indexToName[i]] = i
		else 
			indexToSubGrp[i] = nil
		end
	end  
	
	local subGrpSize = {0,0,0,0,0,0,0,0}
	for i = 1, 40 do
		if indexToSubGrp[i] ~= nil then
			subGrpSize[indexToSubGrp[i]] = subGrpSize[indexToSubGrp[i]] + 1
		end
	end  

	for i = 1, 8 do	
		for j = 1, 5 do
			local currentIndex = (i - 1) * 5 + j 
			if(nameToIndex[sortedGRP[currentIndex]] ~= nil) then
				local actuallIndex = nameToIndex[sortedGRP[currentIndex]]
				
				if(indexToSubGrp[actuallIndex] == i) then
					indexToSubGrp[actuallIndex] = 0
				elseif(subGrpSize[i] < 5) then
					subGrpSize[indexToSubGrp[actuallIndex]] = subGrpSize[indexToSubGrp[actuallIndex]] -1
					
					SetRaidSubgroup(actuallIndex, i)
					
					indexToSubGrp[actuallIndex] = 0
					subGrpSize[i] = subGrpSize[i] + 1
				else 
					for i2 = 1, 40 do
						if(indexToSubGrp[i2] == i) then
							SwapRaidSubgroup(i2, actuallIndex)
							indexToSubGrp[i2] = indexToSubGrp[actuallIndex]
							break
						end
					end
					indexToSubGrp[actuallIndex] = 0
				end
			end
		end
	end
end



RGO_Console:RegisterChatCommand("rgo", "CreateRGOWindow")

function RGO_Console:CreateRGOWindow(input)
    if not RGO:IsOpen() then
        RGO:CreateWindow()
    else
        RGO:closeWindow()
    end
end

